function [Kc,Kf,Q,SS_OUT]=obcanon(plant)
%
%==========================================================
% Observer-Based Realization of a given system on the canonic
% system (with same order).
%==========================================================
%
% [KC,KF,Q,SS_OUT] = OBCANON(PLANT) compute a real Observer
% Based Realization, that is the Youla parameterization
% (defined by Kc, Kf and Q), of a given continuous-time
% system PLANT on a canonical system SYS0 of same order
% (Q is static): 
% The canonical system SYSO is defined by:
%   SYSO=ss([zeros(n,1) [eye(n-1);zeros(1,n-1)]],[zeros(n-p,p);eye(p)],...
%            [eye(m) zeros(m,n-m)],zeros(m,p));
%   where n, m, p are the order, the number of imputs and the number of
%   outputs of PLANT. That is: SYSO is a set on n integrators in series
%   with:
%          * the fisrt m intergrators are connected to the m imputs
%          * ouputs of the p last intregrators are connected to the p
%            ouputs.
%
% Remarks: * PLANT and SS_OUT are defined as SS object,
%          * The system PLANT is assumed to be minimal.
%          * Auto-conjugate eigenvalues are assigned together.
%
% SS_OUT is the new state space realization of PLANT
%
% See also: OBR2COR, COR2TFG, COR2TFGA, COR2OBR, COR2OBRA

% D. Alazard 01/99
% revised: 11/06
% revised: 11/10 to handle SS objects
% Copyright (c) 1993-2000 ONERA/DCSD, All Rights Reserved.

% Closed-loop matrix computation:

n=size(plant.a,1);
[p,m]=size(plant.d);
SYSO=ss([zeros(n,1) [eye(n-1);zeros(1,n-1)]],[zeros(n-p,p);eye(p)],...
        [eye(m) zeros(m,n-m)],zeros(m,p));
[T,F,G]=cor2tfga(SYSO,plant);
Kc=-(plant.c*T+plant.d*SYSO.c);
Kf=inv(T)*plant.b-SYSO.b*plant.d;
Q=plant.d;
SS_OUT=obr2cor(SYSO,Kc,Kf,Q);

