function [COR] = obr2cor2ddl(plant,K,G,Q,xr,ur);
% [COR]=OBR2COR2DDL(PLANT,K,G,Q,Xr,Ur) 
% compute the state space realization SYS_K=ss(AK,BK,CK,DK) 
% of the 2 degree-of-freedom observer-based controller defined by the 
% Youla parameterization:
%    - K: state feedback gain, 
%    - G: state estimation gain, 
%    - Q(s): dynamic Youla parameter,
%  (this parameterization concern the plant defined by PLANT)
%    - Xr, Ur are the plant state and input equilibrium condition
%      to reach.
%
% Inputs of SYS_K are measurement (y) and reference input (e= the new
% equilibrium point).
%
% PLANT,Q and SYS_K are defined as SS objects.
%
% Let us note: [A,B,C,D]=ssdata(PLANT), then the controller 
% realization is:
%                                                  _____
%             _____________________________________| Ur|___
%             |   ________________________         |___|  |
%             |  |                        |               |
%   xHatDot   |  | A-BK-GC+GDK | G | B-GD |               | xHat
%         u <-0--|     -K      | 0 |   I  |<-----  ______ |  y 
%    y-yhat /----|    -C+DK    | I |  -D  |<---0<--|K*Xr|-|- e
%           |    |________________________|  + |+  ------
%           |                                  |
%           |               ______             |
%           |              |      |            |
%           \------------->| Q(s) |------------/
%                          |______|
%
% Remark :  * continuous-time formulation (or predictor form in
%             discrete-time) for the state estimator
%           * positive feeedback
%
% see also:  REGLQGQ, COR2LQG, MON_LQG         12/98

% [SYS_K]=REGLQGQ(PLANT,K,G,Q,H,M) allows to take into account a 
%   static matrix action on xHat and x_Q.

% D. Alazard 01/99
% Copyright (c) 1993-2000 ONERA/DCSD, All Rights Reserved.


[a,b,c,d]=ssdata(plant);
[p,m]=size(d);
if nargin==5, ur=zeros(m,1);end;
COR=obr2cor2ddl(plant,K,G,Q,K*xr);
COR=ss(COR.a,COR.b,COR.c,COR.d+[zeros(size(COR.D)-[0 1]) ur]);

